@@ -1,18 +1,15 @@
 productname = 'OfflineIMAP'
-versionstr = "4.0.13"
-revno = long('$Rev: 592 $'[6:-2])
-revstr = "Rev %d" % revno
-datestr = '$Date: 2004-08-01 16:50:23 -0500 (Sun, 01 Aug 2004) $'
+versionstr = "5.99.4"
 
 versionlist = versionstr.split(".")
 major = versionlist[0]
 minor = versionlist[1]
 patch = versionlist[2]
-copyright = "Copyright (C) 2002 - 2005 John Goerzen"
+copyright = "Copyright (C) 2002 - 2007 John Goerzen"
 author = "John Goerzen"
 author_email = "jgoerzen@complete.org"
 description = "Disconnected Universal IMAP Mail Synchronization/Reader Support"
-bigcopyright = """%(productname)s %(versionstr)s (%(revstr)s)
+bigcopyright = """%(productname)s %(versionstr)s
 %(copyright)s <%(author_email)s>""" % locals()
 
 banner = bigcopyright + """
@@ -20,9 +17,8 @@ This software comes with ABSOLUTELY NO WARRANTY; see the file
 COPYING for details.  This is free software, and you are welcome
 to distribute it under the conditions laid out in COPYING."""
 
-homepage = "http://www.quux.org/devel/offlineimap"
-homegopher = "gopher://quux.org/1/devel/offlineimap"
-license = """Copyright (C) 2002 - 2004 John Goerzen <jgoerzen@complete.org>
+homepage = "http://software.complete.org/offlineimap/"
+license = """Copyright (C) 2002 - 2007 John Goerzen <jgoerzen@complete.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -38,10 +34,16 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA"""
 
-cmdhelp = """
+def getcmdhelp():
+    from offlineimap.ui import detector
+    import os
+    uilist = ""
+    for ui in detector.DEFAULT_UI_LIST:
+        uilist += "                " + ui + os.linesep
+    return """
        offlineimap [ -1 ] [ -P profiledir ] [ -a accountlist ]  [
        -c configfile  ] [ -d debugtype[,debugtype...]  ] [ -o ] [
-       -u interface ]
+       -u interface ] [ -q ]
 
        offlineimap -h | --help
 
@@ -95,6 +97,9 @@ cmdhelp = """
        -o     Run  only once, ignoring any autorefresh setting in
               the config file.
 
+       -q     Run  only quick synchronizations.   Ignore any flag
+              updates on IMAP servers.
+
        -h, --help
               Show summary of options.
 
@@ -104,7 +109,6 @@ cmdhelp = """
               configuration file.  The UI specified with -u  will
               be  forced to be used, even if its isuable() method
               states that it cannot be.   Use  this  option  with
-              care.   The  pre-defined  options are listed in the
-              USER INTERFACES section.
-
-"""
+              care.   The  pre-defined  options, described in the
+              USER INTERFACES section of the man page, are:
+""" + uilist
